from importlib import resources
import json
import logging.config
from abc import ABC, abstractmethod


class LoggerStrategy(ABC):
    @abstractmethod
    def setup_logging(self):
        pass


class InfoLoggerStrategy(LoggerStrategy):
    def setup_logging(self):
        with resources.open_text("bragir.logging_configs", "info.json") as f:
            logging_config = json.load(f)
        logging.config.dictConfig(config=logging_config)


class DebugLoggerStrategy(LoggerStrategy):
    def setup_logging(self):
        with resources.open_text("bragir.logging_configs", "debug.json") as f:
            logging_config = json.load(f)
        logging.config.dictConfig(config=logging_config)



# Constants for file operations

# List of files to be ignored during processing
BLACKLISTED_FILES = [".DS_Store"]

# Maximum number of tokens for transcription
TOKEN_LIMIT = 1500

# Bitrates in kbps for different audio file formats
BITRATES_KBPS = {
    "mp3": [128, 192, 256, 320],
    "mp4": [96, 128, 160, 192, 256, 320],
    "mpeg": [1000, 1500, 3000, 4000, 10000],
    "mpga": [96, 128, 160, 192, 256, 320],
    "m4a": [128, 160, 192, 256, 320],
    "wav": [1410, 2820],
    "webm": [500, 1000, 1500, 3000, 4000, 10000],
}

# Duration in seconds for a 25MB file with the highest possible kbps for different audio file formats
DURATION_SECONDS_25MB = {
    "mp3": 640.0,
    "mp4": 640.4,
    "mpeg": 2.5,
    "mpga": 2.5,
    "m4a": 320.0,
    "wav": 8.0,
    "webm": 2.5,
}



from enum import StrEnum, auto
from typing import Any, Type


class Languages(StrEnum):
    PORTUGUESE = auto()
    SPANISH = auto()
    FRENCH = auto()
    POLISH = auto()
    GERMAN = auto()
    ITALIAN = auto()
    DUTCH = auto()
    SWEDISH = auto()
    DANISH = auto()
    NORWEGIAN = auto()
    FINNISH = auto()
    TURKISH = auto()


def to_output(enum_class: Type[StrEnum]) -> str:
    formatted_enum = " ".join([f"{enum.value}" for enum in enum_class])
    return formatted_enum


def parse_languages(input_languages: Any):
    valid_languages: list[Languages] = []
    valid_string_languages = [language.value.lower() for language in Languages]

    for target_language in input_languages:
        if target_language.lower() in valid_string_languages:
            valid_languages.append(Languages[target_language.upper()])

    return valid_languages



from dataclasses import dataclass, field
import math
import re

from bragir.constants import TOKEN_LIMIT
from bragir.languages import Languages
from bragir.srt.srt_part import SRTPart


@dataclass
class File:
    name: str
    language: Languages
    source_path: str = ""
    target_path: str = ""
    contents: str = ""
    translated_content: str = ""
    SRTParts: list[SRTPart] = field(default_factory=list[SRTPart])
    breakpoints: list[int] = field(default_factory=list[int])

    @property
    def number_of_tokens(self) -> int:
        words = re.findall(r"\w+|[^\w\s]", self.contents, re.UNICODE)
        return len(words) + self.contents.count(" ")

    @property
    def number_of_chunks(self) -> int:
        return math.ceil(self.number_of_tokens / TOKEN_LIMIT)



from bragir.files.file import File
from bragir.tracing.logger import logger


def read_file(file_path: str):
    with open(file_path, "r") as file:
        file_content = file.read()
    return file_content


def create_file(file: File, content: str):
    logger.info(f"Creating file {file.target_path}")
    with open(file.target_path, "a+", encoding="utf-8") as fileIO:
        fileIO.write(content)
        logger.info(f"Created file {file.target_path}")



import os

import click
from bragir.constants import TOKEN_LIMIT
from bragir.files.file import File
from bragir.languages import Languages
from bragir.audio.chunking import chunk_audio
from bragir.srt.srt_part import SRTPart
from bragir.tracing.logger import logger


def calculate_duration_ms(file_size_mb: int, bitrate_kbps: int) -> float:
    file_size_bits = file_size_mb * 8 * 1024 * 1024  # Convert MB to bits
    duration_ms = (
        file_size_bits / bitrate_kbps
    ) * 1000  # Calculate duration in milliseconds
    return duration_ms


def calculate_file_size(file_path: str) -> float:
    file_size_bytes = os.path.getsize(file_path)
    return file_size_bytes / (1024 * 1024)


def read_file(file_path: str):
    with open(file_path, "r") as file:
        file_content = file.read()
    return file_content


def create_file(file: File, content: str):
    with open(file.target_path, "a+", encoding="utf-8") as fileIO:
        fileIO.write(content)


def get_new_file_path(file: str, target_language: Languages) -> str:
    base_path, file_name = os.path.split(file)
    updated_file_name = f"{target_language.value.lower()[:3]}_{file_name}"
    new_file_path = os.path.join(base_path, updated_file_name)
    return new_file_path


def chunk_content_into_srt_parts(content: str) -> list[SRTPart]:
    blocks = content.strip().split("\n\n")

    SRTParts: list[SRTPart] = []

    for block in blocks:
        lines = block.split("\n")

        number = int(lines[0])
        times = lines[1].split(" --> ")
        start_time, end_time = times[0], times[1]
        text = " ".join(lines[2:])

        SRTParts.append(
            SRTPart(
                index=number,
                start_time=start_time,
                end_time=end_time,
                content=text,
                source=block,
            )
        )

    return SRTParts


def get_breakpoints(SRTParts: list[SRTPart]) -> list[int]:
    buffer_limit: int = TOKEN_LIMIT
    buffer = 0

    breakpoints: list[int] = []

    for i, strpart in enumerate(SRTParts):
        buffer += strpart.number_of_tokens

        if buffer > buffer_limit:
            prev_srt_part = SRTParts[i - 1]
            next_srt_part = strpart

            breakpoints.append(i)

            sentences: list[str] = prev_srt_part.content.split(".")

            last_sentence = sentences[-1]

            prev_srt_part.content = "".join(prev_srt_part.content.split(".")[:-1])

            next_srt_part.content = last_sentence + " " + next_srt_part.content

            buffer = 0

    return breakpoints


def chunk_content(file_content: str) -> tuple[list[SRTPart], list[int]]:
    srt_parts: list[SRTPart] = chunk_content_into_srt_parts(file_content)

    breakpoints: list[int] = get_breakpoints(srt_parts)

    return (srt_parts, breakpoints)


def process_files(file_paths: list[str], languages: list[Languages]) -> list[File]:
    processed_files: list[File] = []
    for file_path in file_paths:
        file_content = read_file(file_path)
        (srt_parts, breakpoints) = chunk_content(file_content)
        for language in languages:
            base_path, file_name = os.path.split(file_path)
            updated_file_name = f"{language.value.lower()[:3]}_{file_name}"
            new_file_path = os.path.join(base_path, updated_file_name)
            file = File(
                name=file_path,
                language=language,
                SRTParts=srt_parts,
                breakpoints=breakpoints,
                target_path=new_file_path,
                source_path=file_path,
                contents=file_content,
            )

            processed_files.append(file)

    return processed_files


def process_file(file_path: str) -> list[str]:
    _, file_extension = os.path.splitext(file_path)
    file_extension = file_extension.lower().replace(".", "")

    chunks = chunk_audio(file_path, format=file_extension)
    directory, base_name = os.path.split(file_path)

    chunk_paths: list[str] = []
    for i, chunk in enumerate(chunks):
        new_base_name = f"{i}_{base_name}"
        new_path = os.path.join(directory, new_base_name)
        chunk.export(new_path, format=file_extension)  # type:ignore
        chunk_paths.append(new_path)  # type:ignore

    return chunk_paths


def remove_files(file_paths: list[str]):
    logger.info("Starting cleanup process...")
    for path in file_paths:
        if not os.path.exists(path):
            logger.info(f"File {path} not found. Skipping.")
            continue

        try:
            os.remove(path)
            logger.info(f"File {path} has been successfully removed.")
        except PermissionError as e:
            raise click.ClickException(
                f"Permission denied while trying to remove {path}: {e}"
            )
        except Exception as e:
            raise click.ClickException(f"Error removing file {path}: {e}")



import logging
import logging.config

from bragir.tracing.stratergies import LoggerStrategy


def setup_logging(strategy: LoggerStrategy) -> None:
    strategy.setup_logging()


logger = logging.getLogger("root")



from configparser import ConfigParser
from enum import StrEnum, auto
import os
from pathlib import Path

import click
from pydantic import BaseModel, ValidationError

from bragir.tracing.logger import logger

CONFIG_DIR = Path.home() / ".bragir" / "cli"
CONFIG_FILE_PATH = CONFIG_DIR / "config.ini"

BASE_CONFIG = """# DONT CHANGE STRUCTURE OF THIS FILE
[audio]
min_silence_len=1000
silence_thresh=-40
keep_silence=True

[logging]
level=info

[client]
openai_api_key=YOUR_API_KEY
"""


class LoggingLevel(StrEnum):
    DEBUG = auto()
    INFO = auto()
    WARNING = auto()
    ERROR = auto()
    CRITICAL = auto()


class AudioConfig(BaseModel):
    min_silence_len: int
    silence_thresh: int
    keep_silence: bool


class LoggingConfig(BaseModel):
    level: LoggingLevel


class ClientConfig(BaseModel):
    openai_api_key: str


class Config(BaseModel):
    audio: AudioConfig
    logging: LoggingConfig
    client: ClientConfig


def create_config_file(target_path: Path):
    user_home_directory = os.path.expanduser("~")

    full_file_path = os.path.join(user_home_directory, ".bragir/cli/config.ini")

    os.makedirs(os.path.dirname(full_file_path), exist_ok=True)

    logger.info(f"Creating config file at: {target_path}")
    with open(target_path, "w") as file:
        file.write(BASE_CONFIG)


def read_config(
    file_path: Path = CONFIG_FILE_PATH,
) -> Config:
    logger.info(f"Reading config from {file_path}")
    config_parser = ConfigParser()
    config_parser.read(file_path)

    values = {
        "audio": {
            "min_silence_len": config_parser.getint("audio", "min_silence_len"),
            "silence_thresh": config_parser.getint("audio", "silence_thresh"),
            "keep_silence": config_parser.getboolean("audio", "keep_silence"),
        },
        "logging": {"level": config_parser.get("logging", "level")},
        "client": {"openai_api_key": config_parser.get("client", "openai_api_key")},
    }

    audio_config = AudioConfig(**values["audio"])  # type: ignore
    logging_config = LoggingConfig(**values["logging"])  # type: ignore
    client_config = ClientConfig(**values["client"])  # type: ignore

    return Config(audio=audio_config, logging=logging_config, client=client_config)


try:
    config = read_config(Path(CONFIG_FILE_PATH).expanduser())
except ValidationError as e:
    click.secho(f"Configuration validation error: {e}", fg="red")



import click

from bragir.client import initiate_client
from bragir.config import LoggingLevel, config
from bragir.tracing.logger import setup_logging
from bragir.tracing.stratergies import DebugLoggerStrategy, InfoLoggerStrategy

from . import commands


CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])


@click.group(context_settings=CONTEXT_SETTINGS)
@click.pass_context
@click.version_option()
@click.option(
    "logging_level",
    "--logging_level",
    "-ll",
    type=click.Choice(["info", "debug"], case_sensitive=False),
    help="Use the log level",
)
def cli(context: click.Context, logging_level: str):
    """
    Bragir is an tool that can generate SRT files from videos and translate SRT files.

    In order to use Bragir, an OpenAI api_key needs to be passed as an option. Or alternativly,
    as an enviroment variable in the current session
    """

    context.ensure_object(dict)

    api_key: str = config.client.openai_api_key

    client = initiate_client(api_key=api_key)

    context.obj["client"] = client

    if not logging_level:
        logging_level = config.logging.level

    logging_level = logging_level.upper()
    if logging_level == "INFO":
        setup_logging(InfoLoggerStrategy())
    elif logging_level == "DEBUG":
        setup_logging(DebugLoggerStrategy())

    config.logging.level = LoggingLevel[logging_level]
    context.obj["config"] = config


cli.add_command(commands.transcribe)
cli.add_command(commands.translate)



import os
from pathlib import Path

from bragir.config import CONFIG_FILE_PATH, create_config_file, read_config
from bragir.tracing.logger import logger

path = Path(CONFIG_FILE_PATH).expanduser()

if not os.path.isfile(path):
    logger.info(f"Config file not found at: {CONFIG_FILE_PATH}")
    logger.info(f"Creating config file at: {CONFIG_FILE_PATH}")
    create_config_file(path)
    logger.info(f"Created config file at: {CONFIG_FILE_PATH}")


config = read_config(path)



import re
from dataclasses import dataclass


@dataclass
class SRTPart:
    index: int
    start_time: str
    end_time: str
    content: str
    source: str = ""

    @property
    def number_of_tokens(self) -> int:
        words = re.findall(r"\w+|[^\w\s]", self.content, re.UNICODE)
        return len(words) + self.content.count(" ")

    @property
    def srt_format(self) -> str:
        return (
            f"{self.index}\n{self.start_time} --> {self.end_time}\n{self.content}\n\n"
        )



from pydub import AudioSegment
from pydub.silence import split_on_silence  # type:ignore

from bragir.config import AudioConfig, config
from bragir.audio.grouping import group_audio_segments
from bragir.constants import DURATION_SECONDS_25MB
from bragir.tracing.logger import logger


def chunk_audio(
    file_path: str,
    format: str = "mp4",
    audio_config: AudioConfig = config.audio,
) -> list[AudioSegment]:
    sound: AudioSegment = AudioSegment.from_file(  # type:ignore
        file=file_path
    )

    logger.info(f"Spliting {file_path} on silence")

    chunks: list[AudioSegment] = split_on_silence(
        sound,
        min_silence_len=audio_config.min_silence_len,
        silence_thresh=audio_config.silence_thresh,
        keep_silence=audio_config.keep_silence,
    )  # type: ignore

    logger.info(f"Number of chunks {len(chunks)} when splitted on silence")

    limit = DURATION_SECONDS_25MB[format] * 0.9

    logger.info(f"Limit {limit} seconds for {format}")

    audio_segments = group_audio_segments(chunks, limit)

    logger.info(f"Number of audio segments {len(audio_segments)}")

    return audio_segments



from openai import OpenAI

from bragir.files.file import File
from bragir.list import split_list_at_breakpoints
from bragir.tracing.logger import logger
from bragir.timer import timing_decorator


def translate_content(client: OpenAI, text: str, language: str) -> str:
    completion = client.chat.completions.create(
        model="gpt-4-1106-preview",
        messages=[
            {
                "role": "system",
                "content": f"You are a translator machine, you can only translate to the following language {language}. You need to keep the exact same format as the file. only translate the pieces of text. Make sure that all the text is translated and that there are no timestamps missing. Don't add any whitespace on the first line of the file or the last line of the file",
            },
            {"role": "user", "content": text},
        ],
    )

    return completion.choices[0].message.content or ""


@timing_decorator
def translate_srt(translator: OpenAI, file: File, language: str) -> str:
    logger.info(f"Translating {file.source_path} to {language}")

    translated_text = ""

    if len(file.breakpoints) == 0:
        translated_text += translate_content(translator, file.contents, language)

    if len(file.breakpoints) > 0:
        chunks = split_list_at_breakpoints(file.SRTParts, breakpoints=file.breakpoints)

        for i, chunk in enumerate(chunks):
            text = ""
            for part in chunk:
                text += part.srt_format
            text.rstrip("\n")

            translated_text += translate_content(translator, text, language)

            # Add new block onto the text
            translated_text += "\n\n"

            logger.info(f"Chunk {i + 1} translated")

    logger.info(f"Translated {file.source_path} to {language}")
    return translated_text



from typing import Any


def split_list_at_breakpoints(original_list: list[Any], breakpoints: list[int]) -> list[list[Any]]:
    if len(original_list) == 0 or len(original_list) == 1:
        return original_list
    
    result: list[Any] = []
    start_index = 0

    for breakpoint in breakpoints:
        result.append(original_list[start_index:breakpoint])
        start_index = breakpoint

    # Append the remaining elements after the last breakpoint
    result.append(original_list[start_index:])

    for res in result:
        if res == []:
            result.remove(res)

    return result


import os
from bragir.tracing.logger import logger
from bragir.constants import BLACKLISTED_FILES


def get_files_in_directory(path: str) -> list[str]:
    paths: list[str] = []

    for root, _dirs, nested_files in os.walk(path):
        for nested_file in nested_files:
            if nested_file not in BLACKLISTED_FILES:
                # Create the full path to the file
                logger.info(f"Adding file {nested_file} for translation")
                paths.append(os.path.join(root, nested_file))

    return paths



import os


def get_target_path(path: str, output: str) -> str:
    if output and os.path.isdir(output):
        root, _ = os.path.splitext(path)
        return os.path.join(output, os.path.basename(root) + ".srt")

    if output and os.path.isfile(output):
        return output

    root, _ = os.path.splitext(path)
    return root + ".srt"



import time
from typing import Callable, Any
from bragir.tracing.logger import logger


def timing_decorator(func: Callable[..., Any]) -> Callable[..., Any]:
    def wrapper(*args: Any, **kwargs: Any) -> Any:
        start_time = time.time()
        result = func(*args, **kwargs)
        end_time = time.time()
        elapsed_time = end_time - start_time
        logger.info(f"{func.__name__} elapsed time: {elapsed_time} seconds")
        return result

    return wrapper



from anyio import Path
from openai import OpenAI
from bragir.tracing.logger import logger
from bragir.file import calculate_file_size, process_file, remove_files

from bragir.timer import timing_decorator


@timing_decorator
def transcribe_audio_files(transcriber: OpenAI, audio_paths: list[str]) -> list[str]:
    return [transcribe_audio(transcriber, path) for path in audio_paths]


def transcribe_audio(client: OpenAI, audio_path: str) -> str:
    logger.info(f"Transcribing {audio_path}")
    transcript = client.audio.transcriptions.create(
        model="whisper-1", file=Path(audio_path), response_format="srt"
    )
    return transcript  # type:ignore


def transcribe_file(transcriber: OpenAI, path: str) -> list[str]:
    tmp_audio_paths: list[str] = []

    print("Transcribing file")

    try:
        file_size_mbytes = calculate_file_size(path)

        if file_size_mbytes < 25:
            return [transcribe_audio(transcriber, path)]

        tmp_audio_paths = [*tmp_audio_paths, *process_file(path)]

        return transcribe_audio_files(transcriber, tmp_audio_paths)
    finally:
        logger.info("Removing temporary files")
        remove_files(tmp_audio_paths)



import os
import click
from openai import OpenAI
from bragir.directory import get_files_in_directory
from bragir.file import (
    chunk_content,
    chunk_content_into_srt_parts,
    get_new_file_path,
    process_files,
)
from bragir.files.file import File
from bragir.files.operations import create_file, read_file
from bragir.languages import Languages, parse_languages
from bragir.spinner import spinner
from bragir.tracing.logger import logger
from bragir.messages import PROMPT_HELP
from bragir.path import get_target_path
from bragir.srt.srt_part import SRTPart
from bragir.time import update_timestamps
from bragir.transcription import transcribe_file
from bragir.translation import translate_srt


@click.command(options_metavar="<options>")
@click.argument(
    "path",
    type=click.Path(exists=True, file_okay=True, dir_okay=True),
    metavar="<path>",
)
@click.argument(
    "output",
    type=click.Path(exists=False, file_okay=True, dir_okay=True, writable=True),
    required=False,
)
@spinner("transcribe")
@click.pass_context
def transcribe(context: click.Context, path: str, output: str) -> None:
    """
    The transcribe command generates an SRT file based on an .mp4 or .mp3 file.
    If output is not set, then it will take the file_path name and change the extension.
    """
    logger.info("Starting transcription")

    path_is_file = os.path.isfile(path)
    path_is_directory = os.path.isdir(path)

    if not path_is_file and not path_is_directory:
        logger.info("Please provide a file or directory")
        exit(1)

    transcriber = context.obj["client"]

    file_paths: list[str] = []

    if path_is_directory:
        directory_file_paths = get_files_in_directory(path)
        file_paths = [*file_paths, *directory_file_paths]

    if path_is_file:
        file_paths.append(path)

    logger.info(f"Starting transcription of {path}")

    for file_path in file_paths:
        transcripts: list[str] = transcribe_file(transcriber, file_path)

        videos_srts: list[tuple[int, list[SRTPart]]] = [
            (order, chunk_content_into_srt_parts(transcript))
            for order, transcript in enumerate(transcripts)
        ]

        sorted_videos = sorted(videos_srts)

        srt_parts: list[SRTPart] = update_timestamps(sorted_videos)

        contents = "".join([srt_part.srt_format for srt_part in srt_parts])

        target_path = get_target_path(file_path, output)

        with open(target_path, "w", encoding="utf-8") as fileIO:
            fileIO.write(contents)
            logger.info(f"Created {target_path} for video {path}")
            click.echo(f"Created {target_path} for video {path}")


@click.command(options_metavar="<options>")
@click.argument(
    "path",
    type=click.Path(dir_okay=True, exists=True),
    metavar="<path>",
)
@click.option(
    "--language",
    "-l",
    required=True,
    type=click.Choice([language.value for language in Languages], case_sensitive=False),
    multiple=True,
    help=PROMPT_HELP["language"],
)
@click.pass_context
@spinner("translate")
def translate(context: click.Context, path: str, language: str) -> None:
    """
    The translate command, translates either a single SRT file or files or directory of SRT files into the wanted language.
    """
    logger.info("Starting translation")

    translator: OpenAI = context.obj["client"]

    path_is_file = os.path.isfile(path)
    path_is_directory = os.path.isdir(path)

    if not path_is_directory and not path_is_file:
        logger.info("Please provide a file or directory")
        exit(1)

    translate_to_languages: list[Languages] = parse_languages(language)

    logger.info(
        f"Translating to following language/languages: {' '.join([language.value for language in translate_to_languages])}"
    )

    files: list[File] = []
    if path_is_file:
        logger.info(f"Adding file {path} for translation")

        for target_language in translate_to_languages:
            logger.info(
                f"Adding file {path} with {target_language.value} for translation"
            )

            target_path = get_new_file_path(path, target_language)

            file_content = read_file(path)

            (srt_parts, breakpoints) = chunk_content(file_content)

            files.append(
                File(
                    name=path,
                    contents=file_content,
                    language=target_language,
                    SRTParts=srt_parts,
                    breakpoints=breakpoints,
                    target_path=target_path,
                    source_path=path,
                )
            )

    if path_is_directory:
        directory_file_paths = get_files_in_directory(path)

        num_of_file_paths = len(directory_file_paths)

        if num_of_file_paths == 1:
            logger.info(f"Processing {num_of_file_paths} file in directory {path}")
        else:
            logger.info(f"Processing {num_of_file_paths} files in directory {path}")

        files = process_files(directory_file_paths, translate_to_languages)

    num_of_file_paths = len(files)

    if num_of_file_paths == 1:
        logger.info(f"Processing {num_of_file_paths} file")
    else:
        logger.info(f"Processing {num_of_file_paths} files")

    for target_file in files:
        translated_content = translate_srt(
            translator, target_file, target_file.language.value
        )
        create_file(target_file, translated_content)

        click.echo(f"Created file {target_file.target_path}")
        logger.info(f"Created file {target_file.target_path}")



from datetime import timedelta
import re
from typing import Tuple
from bragir.tracing.logger import logger

from bragir.srt.srt_part import SRTPart


def to_timestamp(td: timedelta) -> str:
    total_seconds = int(td.total_seconds())
    hours = total_seconds // 3600
    minutes = (total_seconds % 3600) // 60
    seconds = total_seconds % 60

    milliseconds = td.microseconds // 1000

    return f"{hours:02}:{minutes:02}:{seconds:02}.{milliseconds:03}"


def to_timedelta(timestamp: str) -> timedelta:
    parts = re.split("[:,.]", timestamp)
    parts = [float(part) for part in parts]

    if len(parts) == 3:
        return timedelta(hours=parts[0], minutes=parts[1], seconds=parts[2])

    return timedelta(
        hours=parts[0], minutes=parts[1], seconds=parts[2], milliseconds=parts[3]
    )


def update_timestamps(videos_srts: list[Tuple[int, list[SRTPart]]]) -> list[SRTPart]:
    index_pointer = 1
    video_end_time = timedelta(hours=0, minutes=0, seconds=0)
    all_srt_parts: list[SRTPart] = []

    logger.info("Updating timestamps")
    for _order, srt_parts in sorted(videos_srts):
        for srt_part in srt_parts:
            srt_part.start_time = to_timestamp(
                to_timedelta(srt_part.start_time) + video_end_time
            )
            srt_part.end_time = to_timestamp(
                to_timedelta(srt_part.end_time) + video_end_time
            )
            srt_part.index = index_pointer
            index_pointer += 1
            all_srt_parts.append(srt_part)
        video_end_time = to_timedelta(srt_parts[-1].end_time)

    return all_srt_parts
